import {fakeAsync, flush, TestBed} from '@angular/core/testing';

import { ActivitiesService } from './activities.service';
import {LocalizationPipe} from './localize';
import {Subject} from 'rxjs';
import {DataCardService} from '../api/data-card.service';
import {CarePlanService} from '../api/care-plan.service';
import {Activity, CarePlan, DataCard} from '@nspop/gm-web-facade-api';

describe('ActivitiesService', () => {
  let service: ActivitiesService;

  let carePlanSubjectStub: Subject<CarePlan>;
  let carePlanServiceStub: CarePlanService;

  let selectedDataCardSubjectStub: Subject<DataCard>;
  let dataCardServiceStub: DataCardService;

  beforeEach(async () => {
    carePlanSubjectStub = new Subject<CarePlan>();
    carePlanServiceStub = {
      carePlan$: carePlanSubjectStub.asObservable(),
      update: (carePlan: CarePlan, id: string, version: string) => ({ subscribe: () => {}}),
    } as any;

    selectedDataCardSubjectStub = new Subject<DataCard>();
    dataCardServiceStub = {
      selectedDataCard$: selectedDataCardSubjectStub.asObservable(),
    } as any;

    await TestBed.configureTestingModule({
      declarations: [LocalizationPipe],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: CarePlanService, useValue: carePlanServiceStub},
      ]
    })
      .compileComponents();
  });
  beforeEach(() => {
    TestBed.configureTestingModule({});
    service = TestBed.inject(ActivitiesService);
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('filters past activities', fakeAsync(() => {
    let calledTimes = 0;
    service.pastActivities$.subscribe((activities: Activity[]) => {
      calledTimes++;
      expect(activities.length).toBe(1);
      expect(activities[0].status).toBe('finished');
    });

    carePlanSubjectStub.next({
      activities: [
        {status: 'finished'},
        {status: 'expected'},
      ],
    } as CarePlan);

    flush();
    expect(calledTimes).toBe(1);
  }));

  it('filters future activities', () => {
    let calledTimes = 0;
    service.futureActivities$.subscribe((activities: Activity[]) => {
      calledTimes++;
      expect(activities.length).toBe(1);
      expect(activities[0].status).toBe('expected');
    });

    carePlanSubjectStub.next({
      activities: [
        {status: 'finished'},
        {status: 'expected'},
      ],
    } as CarePlan);

    expect(calledTimes).toBe(1);
  });

  it('publishes an empty array if carePlan.activities is undefined', () => {
    let calledTimes = 0;
    service.futureActivities$.subscribe((activities: Activity[]) => {
      calledTimes++;
      expect(activities.length).toBe(0);
    });

    carePlanSubjectStub.next(undefined);

    expect(calledTimes).toBe(1);
  });

  it('returns a copy of the published carePlan', () => {
    const carePlan = {
      activities: [
        {status: 'finished'},
        {status: 'expected'},
      ],
    } as CarePlan;
    carePlanSubjectStub.next(carePlan);

    expect(service.carePlan).toEqual(carePlan);
  });

  it('returns a copy of the published dataCard', () => {
    const dataCard = {
      documentUniqueId: 123,
      version: 456,
    } as any as DataCard;
    selectedDataCardSubjectStub.next(dataCard);

    expect(service.dataCard).toEqual(dataCard);
  });

  it('cancels an activity', fakeAsync(() => {
    spyOn(service.carePlanService, 'update').and.callThrough();

    const activity = {status: 'expected', id: 0} as any as Activity;
    const carePlan = {
      id: '123',
      version: 456,
      activities: [
        activity,
      ],
    } as any as CarePlan;

    carePlanSubjectStub.next(carePlan);
    service.cancelActivity(activity);

    const updatedCarePlan = {
      id: '123',
      version: 456,
      activities: [
        {status: 'cancelled', id: 0},
      ],
    } as any as CarePlan;

    expect(service.carePlanService.update).toHaveBeenCalledTimes(1);
    expect(service.carePlanService.update).toHaveBeenCalledWith(updatedCarePlan, '123', '456');
  }));

  it('does not cancel an activity if it is not found', fakeAsync(() => {
    spyOn(service.carePlanService, 'update').and.callThrough();

    const activity = {status: 'expected', id: 0} as any as Activity;
    const carePlan = {
      id: '123',
      version: 456,
      activities: [
        {status: 'expected', id: 1},
      ],
    } as any as CarePlan;

    carePlanSubjectStub.next(carePlan);
    service.cancelActivity(activity);

    expect(service.carePlanService.update).toHaveBeenCalledTimes(0);
  }));

  it('does not cancel an activity if the activities id is undefined', fakeAsync(() => {
    spyOn(service.carePlanService, 'update').and.callThrough();

    const activity = {status: 'expected'} as any as Activity;
    const carePlan = {
      id: '123',
      version: 456,
      activities: [
        {status: 'expected', id: 1},
      ],
    } as any as CarePlan;

    carePlanSubjectStub.next(carePlan);
    service.cancelActivity(activity);

    expect(service.carePlanService.update).toHaveBeenCalledTimes(0);
  }));

  it('does not cancel an activity if the carePlan.activities is undefined', fakeAsync(() => {
    spyOn(service.carePlanService, 'update').and.callThrough();

    const activity = {status: 'expected', id: 0} as any as Activity;
    const carePlan = {
      id: '123',
      version: 456,
    } as any as CarePlan;

    carePlanSubjectStub.next(carePlan);
    service.cancelActivity(activity);

    expect(service.carePlanService.update).toHaveBeenCalledTimes(0);
  }));

  it('publishes openActivityModalEvent with the given activity', () => {
    let calledTimes = 0;
    const activity = { status: 'expected' } as any as Activity;

    service.openActivityModalEvent$.subscribe((a: Activity) => {
      calledTimes++;
      expect(a).toEqual(activity);
    });

    service.emitOpenActivityModalEvent(activity);

    expect(calledTimes).toBe(1);
  });

  it('publishes openNoteModalEvent with the given activity', () => {
    let calledTimes = 0;
    const activity = { status: 'expected' } as any as Activity;

    service.openNoteModalEvent$.subscribe((a: Activity) => {
      calledTimes++;
      expect(a).toEqual(activity);
    });

    service.emitOpenNoteModalEvent(activity);

    expect(calledTimes).toBe(1);
  });
});
