import {Component, OnDestroy, OnInit, ViewChild} from '@angular/core';
import {ModalComponent} from '../../modals/modal/modal.component';
import {
  FormBuilder,
  FormGroup,
  FormGroupDirective,
  Validators
} from '@angular/forms';
import {Activity, ActivityPlannedTime, CarePlan, DataCard} from '@nspop/gm-web-facade-api';
import {calculateFromAndToWeek, calculateWeekNumberFromPregnancyStart} from '../../../utils/dates';
import {ActivitiesService} from '../activities.service';
import {copy} from '../../../utils/copy';
import {CarePlanService} from '../../api/care-plan.service';
import {ActivityModalErrorStateMatcher, ActivityModalValidator} from './activity-modal.validation';

@Component({
  selector: 'app-activity-modal',
  templateUrl: './activity-modal.component.html',
  styleUrls: ['./activity-modal.component.scss']
})
export class ActivityModalComponent implements OnInit, OnDestroy {
  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('formDirective') public formDirective: FormGroupDirective | undefined;
  public form: FormGroup | undefined;

  public carePlan: CarePlan | undefined;
  public activity: Activity | undefined;
  public dataCard: DataCard | undefined;

  public isEditMode: boolean | undefined;
  public isNoteDirty: boolean = false;

  public errorStateMatcher = new ActivityModalErrorStateMatcher();
  public submitError = false;

  constructor(
    private fb: FormBuilder,
    public activitiesService: ActivitiesService,
    public carePlanService: CarePlanService,
  ) { }

  private readonly MAX_WEEK = 53;

  ngOnInit(): void {
    this.setFormValues();
  }

  ngOnDestroy(): void {
    this.form = undefined;
  }

  public submit(form: FormGroup | undefined, formDirective: FormGroupDirective | undefined): void {
    if (!this.dataCard || !this.carePlan || !this.carePlan.activities) {
      this.submitError = true;
      return;
    }

    if (form?.valid) {
      const activity = this.getActivityFromForm();

      let oldActivity: Activity | undefined;
      let activityIndex: number;

      if (this.activity) {
        activityIndex = this.carePlan.activities.findIndex((a) => a.id === this.activity?.id && a.id !== undefined);

        if (activityIndex === -1) {
          this.submitError = true;
          return console.error('Could not find activity with id: ' + this.activity.id);
        }

        oldActivity = copy(this.carePlan.activities[activityIndex]); // Take copy for reverting purposes

        if (activity.note) {
          activity.note.version = this.activity.note?.version;
          activity.note.documentUniqueId = this.activity.note?.documentUniqueId;
        }

        Object.assign(this.activity, activity); // Write new values, while keeping the old ones for other fields.

        this.carePlan.activities[activityIndex] = this.activity;
      } else {
        activityIndex = this.carePlan.activities.push(activity); // Returns new length
      }

      this.submitCarePlan(this.carePlan, activityIndex, oldActivity, formDirective);
    } else {
      form?.markAllAsTouched();
    }
  }

  private submitCarePlan(carePlan: CarePlan, activityIndex: number, oldActivity?: Activity, formDirective?: FormGroupDirective): void {
    this.carePlanService.update(carePlan, carePlan.id + '', carePlan.version + '').subscribe(
      () => {
        formDirective?.resetForm();
        this.setVisible(false);
        this.carePlanService.refresh();
      },
      (error) => {
        if (this.carePlan?.activities) {
          if (oldActivity) {
            this.activity = oldActivity;
            this.carePlan.activities[activityIndex] = oldActivity;
          } else {
            this.carePlan.activities = this.carePlan.activities.slice(0, activityIndex ? activityIndex - 1 : undefined);
          }
          this.submitError = true;
          console.error(error);
        }
      }
    );
  }

  setActivity(activity: Activity | undefined): void {
    this.activity = copy(activity);
    this.isEditMode = !!activity;
  }

  setVisible(isVisible: boolean): void {
    this.submitError = false;
    this.carePlan = this.activitiesService.carePlan;
    this.dataCard = this.activitiesService.dataCard;
    this.isNoteDirty = false;
    if (this.modalComponent) {
      this.modalComponent.setVisible(isVisible);
      this.formDirective?.resetForm();
      this.setFormValues();
    }
  }

  isVisible(): boolean {
    return !!this.modalComponent?.isModalVisible;
  }

  setFormValues(): void {
    const from = calculateWeekNumberFromPregnancyStart(this.dataCard, this.activity?.plannedTime?.from);
    const to = calculateWeekNumberFromPregnancyStart(this.dataCard, this.activity?.plannedTime?.to);

    this.form = this.fb.group({
      title: [this.activity?.title || '', Validators.required],
      description: [this.activity?.description || '', Validators.required],
      status: [this.activity?.status || 'expected'],
      fromWeek: [from || '', [Validators.required, Validators.min(0), Validators.max(this.MAX_WEEK)]],
      toWeek: [to || '', [Validators.required, Validators.min(0), Validators.max(this.MAX_WEEK)]],
      note: [this.activity?.note?.content || ''],
    }, {validators: [ActivityModalValidator.weekRangeValidator('fromWeek', 'toWeek')]});
  }

  getActivityFromForm(): Activity {
    const {title, description, status, fromWeek, toWeek, note} = this.form?.value;
    const plannedTime: ActivityPlannedTime = calculateFromAndToWeek(this.dataCard, fromWeek, toWeek);
    return {
      title,
      description,
      status,
      plannedTime,
      note: !!note ? { content: note, dirty: this.isNoteDirty} : null, // Only add note if content is a non-empty string
    } as Activity;
  }

  isFromWeekBeforeToWeek(): boolean {
    const isFromWeekFilled = !this.form?.get('fromWeek')?.errors?.required;
    const isToWeekFilled =  !this.form?.get('toWeek')?.errors?.required;
    const isWeekRangeInvalid = !!this.form?.errors?.weekRangeInvalid;

    return isFromWeekFilled && isToWeekFilled && isWeekRangeInvalid;
  }

  isFromWeekNotFilled(): boolean {
    return !!this.form?.get('fromWeek')?.errors?.required;
  }

  public onNoteContentChanged(): void {
    this.isNoteDirty = true;
  }
}
