import {Component, OnInit, ViewChild} from '@angular/core';
import {Activity, CarePlan} from '@nspop/gm-web-facade-api';
import {ModalComponent} from '../../modals/modal/modal.component';
import {ActivitiesService} from '../activities.service';
import {copy} from '../../../utils/copy';
import {CarePlanService} from '../../api/care-plan.service';

export class ActivityWrapper {
  private isSelected: boolean;
  public editable: boolean;

  private defaultStatus: Activity.StatusEnum;

  constructor(public activity: Activity) {
    // Selected activities are those whose status aren't cancelled.
    this.isSelected = activity.status !== 'cancelled';

    // Editable activities are those who are not owned by another user or are already finished.
    this.editable = !(activity.ownership === 'otherUser' || activity.status === 'finished');

    // If status is cancelled, then we set default to 'expected', otherwise use current status.
    this.defaultStatus = this.activity.status === 'cancelled' ? 'expected' : this.activity.status;
  }

  get selected(): boolean {
    return this.isSelected;
  }

  set selected(value: boolean) {
    this.isSelected = value;
    this.setActivityStatus();
  }

  private setActivityStatus(): void {
    if (!this.selected) {
      this.activity.status = 'cancelled';
    }

    // If activity is selected and has status = cancelled, it must mean that the activity was unticked before. Thus we set it to old status.
    if (this.selected && this.activity.status === 'cancelled') {
      this.activity.status = this.defaultStatus;
    }
  }
}

@Component({
  selector: 'app-edit-plan-modal',
  templateUrl: './edit-plan-modal.component.html',
  styleUrls: ['./edit-plan-modal.component.scss']
})
export class EditPlanModalComponent implements OnInit {

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;

  public carePlan: CarePlan | undefined;
  public activities: ActivityWrapper[] = [];
  public submitError: boolean = false;

  constructor(
    private activitiesService: ActivitiesService,
    private carePlanService: CarePlanService,
  ) {
  }

  ngOnInit(): void {
  }

  public setVisible(isVisible: boolean): void {
    this.submitError = false;
    this.carePlan = this.activitiesService.carePlan;
    this.wrapCarePlanActivities();
    this.modalComponent?.setVisible(isVisible);
  }

  public wrapCarePlanActivities(): void {
    this.activities = this.carePlan?.activities?.map((activity: Activity) => new ActivityWrapper(activity)) || [];
  }

  isVisible(): boolean {
    return !!this.modalComponent?.isModalVisible;
  }

  public submit(): void {
    if (!this.carePlan?.activities) {
      this.submitError = true;
      return;
    }
    this.carePlanService.update(this.carePlan, this.carePlan.id + '', this.carePlan.version + '').subscribe(
      () => {
        this.modalComponent?.setVisible(false);
        this.carePlanService.refresh();
      },
      (error) => {
        this.submitError = true;
        console.error(error);
      });
  }

  toggleAll(): void {
    const newSelectionValue = copy(!this.allEditableSelected());
    this.editableActivities.forEach((a: ActivityWrapper) => {
      a.selected = newSelectionValue;
    });
  }

  get editableActivities(): ActivityWrapper[] {
    return this.activities.filter((a: ActivityWrapper) => a.editable);
  }

  get selectedEditableActivities(): ActivityWrapper[] {
    return this.editableActivities.filter((a: ActivityWrapper) => a.selected && a.editable);
  }

  public allEditableSelected(): boolean {
    return this.selectedEditableActivities.length === this.editableActivities.length;
  }

  public someButNotAllEditableSelected(): boolean {
    return this.selectedEditableActivities.length > 0 && !this.allEditableSelected();
  }
}
