import {ComponentFixture, TestBed} from '@angular/core/testing';
import {Subject, throwError} from 'rxjs';
import {Activity, CarePlan} from '@nspop/gm-web-facade-api';
import {Component, Input} from '@angular/core';
import {FormsModule, ReactiveFormsModule} from '@angular/forms';
import {AngularMaterialModule} from '../../modules/angular-material.module';
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {NoteModalComponent} from './note-modal.component';
import {ActivitiesService} from '../activities.service';
import {CarePlanService} from '../../api/care-plan.service';

describe('NoteModalComponent', () => {
  let component: NoteModalComponent;
  let fixture: ComponentFixture<NoteModalComponent>;

  let activitiesServiceStub: ActivitiesService;
  let carePlanSubjectStub: Subject<CarePlan>;
  let carePlanUpdateSubscribeStub: { subscribe: (next: () => void, error: () => void) => void};
  let carePlanServiceStub: CarePlanService;

  @Component({selector: 'app-modal', template: '<div><ng-content></ng-content></div>'})
  class ModalComponent {
    @Input() public header = '';
    private isModalVisible = false;

    public setVisible(isVisible: boolean): void {
      this.isModalVisible = isVisible;
    }
  }

  beforeEach(async () => {
    activitiesServiceStub = {
      carePlan: {
        id: '123',
        version: 456,
        activities: [
          { id: '1', version: 1, status: 'expected'},
          { id: '2', version: 2, status: 'planned'},
        ]
      },
    } as any;

    carePlanSubjectStub = new Subject<CarePlan>();
    carePlanUpdateSubscribeStub =
    carePlanServiceStub = {
      carePlan$: carePlanSubjectStub.asObservable(),
      update: (carePlan: CarePlan, id: string, version: string) => carePlanSubjectStub.asObservable(),
      refresh: () => {
      },
    } as any;

    await TestBed.configureTestingModule({
      declarations: [NoteModalComponent, ModalComponent],
      imports: [
        ReactiveFormsModule,
        FormsModule,
        ReactiveFormsModule,
        AngularMaterialModule,
        BrowserAnimationsModule,
      ],
      providers: [
        {provide: ActivitiesService, useValue: activitiesServiceStub},
        {provide: CarePlanService, useValue: carePlanServiceStub},
      ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(NoteModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('closes modal if submitting carePlan is a success', () => {
    const activity = { id: '1', status: 'expected'} as any as Activity;
    component.activity = activity;

    component.setVisible(true);
    component.carePlan = {id: '123', version: 456, activities: [ activity ]} as any as CarePlan;
    component.form?.setValue({title: 'Ultralydscanning', content: 'vi scanner og alt det der'});

    component.submit(component.form, component.formDirective);
    carePlanSubjectStub.next();
    expect(component.isVisible()).toBe(false);
  });

  it('does not close modal if submitting carePlan fails', () => {
    spyOn(carePlanServiceStub, 'update').and.returnValue(throwError('error'));
    const activity = { id: '1', status: 'expected'} as any as Activity;
    component.activity = activity;

    component.setVisible(true);
    component.carePlan = {id: '123', version: 456, activities: [ activity ]} as any as CarePlan;
    component.form?.setValue({title: 'Ultralydscanning', content: 'vi scanner og alt det der'});

    component.submit(component.form, component.formDirective);
    expect(component.isVisible()).toBe(true);
  });

  it('does not submit if there is no activity set', () => {
    spyOn(carePlanServiceStub, 'update');
    component.activity = undefined;

    component.setVisible(true);
    component.carePlan = {id: '123', version: 456, activities: [ {status: 'expected', id: '1'}]} as any as CarePlan;
    component.form?.setValue({title: 'Ultralydscanning', content: 'vi scanner og alt det der'});

    component.submit(component.form, component.formDirective);
    expect(carePlanServiceStub.update).toHaveBeenCalledTimes(0);
  });

  it('does not submit if there is no careplan set', () => {
    spyOn(carePlanServiceStub, 'update');
    component.activity = {status: 'expected', id: '1'} as Activity;

    component.setVisible(true);
    component.carePlan = undefined;
    component.form?.setValue({title: 'Ultralydscanning', content: 'vi scanner og alt det der'});

    component.submit(component.form, component.formDirective);
    expect(carePlanServiceStub.update).toHaveBeenCalledTimes(0);
  });

  it('does not submit if form is invalid', () => {
    spyOn(carePlanServiceStub, 'update');
    component.activity = {status: 'expected', id: '1'} as Activity;

    component.setVisible(true);
    component.carePlan = {id: '123', version: 456, activities: [ {status: 'expected', id: '1'}]} as any as CarePlan;
    component.form?.setValue({title: 'Ultralydscanning', content: ''});

    component.submit(component.form, component.formDirective);
    expect(carePlanServiceStub.update).toHaveBeenCalledTimes(0);
  });

  it('does not submit if activity is not found in careplan', () => {
    spyOn(carePlanServiceStub, 'update');
    component.activity = {status: 'expected', id: '1'} as Activity;

    component.setVisible(true);
    component.carePlan = {id: '123', version: 456, activities: [ {status: 'expected', id: '2'}]} as any as CarePlan;
    component.form?.setValue({title: 'Ultralydscanning', content: 'vi scanner og alt det der'});

    component.submit(component.form, component.formDirective);
    expect(carePlanServiceStub.update).toHaveBeenCalledTimes(0);
  });

  it('sets editMode = true if there is a note attached to the activity, when setting Activity', () => {
    expect(component.isEditMode).toBeFalsy();

    const activity = {status: 'expected', id: '1', note: {content: 'vi scanner og alt det der'}} as Activity;
    component.setActivity(activity);

    expect(component.isEditMode).toBe(true);
  });

  it('sets editMode = false if there is no note attached to the activity, when setting Activity', () => {
    const activityWithNote = {status: 'expected', id: '1', note: {content: 'vi scanner og alt det der'}} as Activity;
    component.setActivity(activityWithNote);
    expect(component.isEditMode).toBe(true);

    const activityWithoutNote = {status: 'planned', id: '2'} as Activity;
    component.setActivity(activityWithoutNote);
    expect(component.isEditMode).toBe(false);
  });
});
