import {Component, ElementRef, HostListener, OnInit, ViewChild} from '@angular/core';
import {UserContextService} from './services/user-context.service';
import {RoleSelectorComponent} from './role-selector/role-selector.component';
import {MatIconRegistry} from '@angular/material/icon';
import {DomSanitizer} from '@angular/platform-browser';
import {DataCard} from '@nspop/gm-web-facade-api';
import {DialogService} from './modals/dialog/dialog.service';
import {PatientContextService} from './services/patient-context.service';
import {WindowService} from './services/window.service';
import {PartialDataCardError} from './services/error.service';
import {first} from 'rxjs/operators';
import {DataCardService} from './api/data-card.service';
import {CarePlanService} from './api/care-plan.service';
import {ObservationService} from './api/observation.service';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss'],
})
export class AppComponent implements OnInit{
  title = 'Graviditetsportalen';
  dataCard: DataCard | undefined;
  allDataCards: DataCard[] = [];
  hasPatientInContext = false;
  hasEpisodeOfCareInContext = false;
  hasXBreakTheGlassReasonInContext = false;
  hasPrivatelyMarkedDataInContext = false;
  patientExists = false;
  feedbackModalOpen = false;
  prevEpisodeOfCareIdentifier: string | undefined = undefined;

  @ViewChild('sidebar') sidebar?: ElementRef<HTMLElement>;

  constructor(
    public dataCardService: DataCardService,
    public carePlanService: CarePlanService,
    public observationService: ObservationService,
    public userContextService: UserContextService,
    public dialogService: DialogService,
    private patientContextService: PatientContextService,
    private matIconRegistry: MatIconRegistry,
    private domSanitizer: DomSanitizer,
    private windowService: WindowService,
  ) {}

  public ngOnInit(): void {
    this.registerIcons();
    this.registerSmileys();
    // Make sure that userContext has loaded before loading anything else.
    // first() pipe unsubscribes after first value is received, so we don't 'over-subscribe'.
    this.userContextService.userContext$.pipe(first()).subscribe(() => {

        // Business logic
        this.setupDataCardSubscription();
        this.setupCprSubscription();
        this.setupEpisodeOfCareSubscription();

        // Variables for UI rendering
        this.setupXBreakTheGlassReasonSubscription();
        this.setupPrivatelyMarkedSubscription();
        this.setupPatientExistsSubscription();

        // Manual refresh
        this.refreshDataIfCprIsSet();
      }
    );

  }

  private registerIcons(): void {
    this.matIconRegistry.addSvgIcon('ic-add',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_add.svg'));
    this.matIconRegistry.addSvgIcon('ic-add-colored',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_add-colored.svg'));
    this.matIconRegistry.addSvgIcon('ic-calendar',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_calendar.svg'));
    this.matIconRegistry.addSvgIcon('ic-care-plan',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_fagperson-mand.svg'));
    this.matIconRegistry.addSvgIcon('ic-close',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_close-large.svg'));
    this.matIconRegistry.addSvgIcon('ic-close-small-faded',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_close-small-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-close-small-highlighted',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_close-small-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-edit',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_edit.svg'));
    this.matIconRegistry.addSvgIcon('ic-edit-highlight',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_edit-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-info',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_info.svg'));
    this.matIconRegistry.addSvgIcon('ic-lock',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_lock.svg'));
    this.matIconRegistry.addSvgIcon('ic-more',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_more.svg'));
    this.matIconRegistry.addSvgIcon('ic-overview',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_overblik.svg'));
    this.matIconRegistry.addSvgIcon('ic-pdf-highlighted',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_pdf-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-pdf-faded',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_pdf-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-search-faded',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_search-faded.svg'));
    this.matIconRegistry.addSvgIcon('ic-search-highlighted',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_search-highlighted.svg'));
    this.matIconRegistry.addSvgIcon('ic-trash',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_trash.svg'));
    this.matIconRegistry.addSvgIcon('ic-view',
      this.domSanitizer.bypassSecurityTrustResourceUrl('assets/img/ic_view-faded.svg'));
  }

  private registerSmileys(): void {
    const addSmiley = (name: string) => {
      this.matIconRegistry.addSvgIcon(`smiley-${name}`,
        this.domSanitizer.bypassSecurityTrustResourceUrl(`assets/img/smileys/smiley-${name}.svg`));
      this.matIconRegistry.addSvgIcon(`smiley-${name}@faded`,
        this.domSanitizer.bypassSecurityTrustResourceUrl(`assets/img/smileys/smiley-${name}@faded.svg`));
    };
    addSmiley('super-sad');
    addSmiley('sad');
    addSmiley('meh');
    addSmiley('happy');
    addSmiley('super-happy');
  }

  /**
   * Upon receiving dataCards from the BFF we update the episodeOfCare, and do specific actions depending on status.
   * @private
   */
  private setupDataCardSubscription(): void {
    this.dataCardService.allDataCards$.subscribe((dataCards: DataCard[]) => {
      this.allDataCards = dataCards;
      this.dataCard = dataCards?.[0];
      switch (dataCards?.[0]?.documentStatus) {
        case 'ACTIVE':
          this.patientContextService.setEpisodeOfCare(dataCards[0].episodeOfCareIdentifier);
          break;
        case 'NEW':
          // TODO: add handlers here
          break;
        case 'CLOSED':
          // TODO: add handlers here
          break;
        case 'PARTIAL':
          this.patientContextService.setEpisodeOfCare(dataCards[0].episodeOfCareIdentifier);
          const doesNotHaveConsent = !this.patientContextService.getCurrentXBreakTheGlassReason();
          if (doesNotHaveConsent) {
            throw new PartialDataCardError();
          }
          break;
      }
    });
  }

  /**
   * Every time a new cpr is set, we call the server to get the new datacards for that cpr.
   * This is the only way DataCards are updated automatically in the app.
   * @private
   */
  private setupCprSubscription(): void {
    this.patientContextService.cpr$.subscribe((cpr) => {
      this.hasPatientInContext = !!cpr;
      if (cpr) {
        this.dataCardService.refresh();
      }
    });
  }

  /**
   * Every time a new datacard is received, it will set the episodeOfCare, which results calling the server to get a new CarePlan.
   * This is the only way CarePlan is updated automatically in the app.
   * @private
   */
  private setupEpisodeOfCareSubscription(): void {
    this.hasEpisodeOfCareInContext = !!this.patientContextService.getCurrentEpisodeOfCare();
    this.patientContextService.episodeOfCare$.subscribe((episodeOfCareId) => {
      this.hasEpisodeOfCareInContext = !!episodeOfCareId;
      this.dataCard = this.allDataCards.find(x => x.episodeOfCareIdentifier === episodeOfCareId);
      this.carePlanService.refresh();
      this.observationService.refresh();
    });
  }

  private setupXBreakTheGlassReasonSubscription(): void {
    this.hasXBreakTheGlassReasonInContext = !!this.patientContextService.getCurrentXBreakTheGlassReason();
    this.patientContextService.xBreakTheGlassReason$.subscribe((reason => {
      this.hasXBreakTheGlassReasonInContext = !!reason;
    }));
  }

  private setupPrivatelyMarkedSubscription(): void {
    this.hasPrivatelyMarkedDataInContext = !!this.patientContextService.getCurrentHasPrivatelyMarkedData();
    this.patientContextService.hasPrivatelyMarkedData$.subscribe((value => {
      this.hasPrivatelyMarkedDataInContext = !!value;
    }));
  }

  private setupPatientExistsSubscription(): void {
    // If patientExists is undefined, we don't know yet if the patient exists - proceed as if it did
    this.patientExists = this.patientContextService.getCurrentPatientExists() === undefined
      || !!this.patientContextService.getCurrentPatientExists();
    this.patientContextService.patientExists$.subscribe((exists) => {
      this.patientExists = exists === undefined || exists;
    });
  }

  private refreshDataIfCprIsSet(): void {
    this.hasPatientInContext = !!this.patientContextService.getCurrentCpr();
    if (this.hasPatientInContext) {
      this.dataCardService.refresh();
    }
  }

  openChangeRoleDialog(): void {
   this.dialogService.show(RoleSelectorComponent, {
      width: 'fit-content',
    });

  }

  @HostListener('window:scroll')
  public allowYScrollFrame(): void {
    const {x, y} = this.windowService.getXYScroll();
    const style = this.sidebar?.nativeElement.style;
    if (style) {
      style.left = `${-x}px`;
      style.top = `${0}px`;
      if (y <= 72) {
        style.top = `${72 - y}px`;
      }
    }
  }

  openFeedbackDialog(): void {
    this.feedbackModalOpen = true;
  }

  public closePatient(): void {
    console.log('TODO: this should close the patient');
  }
}
