import {
  Component,
  ContentChildren,
  forwardRef,
  Input,
  OnChanges,
  QueryList, SimpleChanges,
  ViewChild
} from '@angular/core';
import {ControlValueAccessor, NG_VALUE_ACCESSOR} from '@angular/forms';
// @ts-ignore
import deepEqual from 'deep-equal';
import {MatSelect} from '@angular/material/select';
import {MatOption} from '@angular/material/core';
import {DataCardService} from '../api/data-card.service';

export const APP_DROPDOWN_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => DropdownComponent),
  multi: true
};

@Component({
  selector: 'app-dropdown',
  templateUrl: './dropdown.component.html',
  styleUrls: ['./dropdown.component.scss'],
  providers: [APP_DROPDOWN_CONTROL_VALUE_ACCESSOR],
  host: {
    '(mouseenter)': 'onMouseEnter()',
    '(mouseleave)': 'onMouseLeave()',
    '(blur)': 'onBlur()',
  }
})
export class DropdownComponent implements ControlValueAccessor, OnChanges {
  @Input() label = '';
  @Input() name = '';
  @Input() multiple = false;
  @Input() notRelevant = false;

  @ViewChild(MatSelect) select: MatSelect|undefined;
  @ContentChildren(MatOption, {descendants: true}) options: QueryList<MatOption>|undefined;

  public innerModel = '';
  public disabled = false;
  public hover = false;

  // When notRelevant is toggled on, we store the old value, so we can re-fill it it gets turned off again
  private oldValue: any;
  private dirtyAble = false;
  private onChangeCallback: (newValue: string) => any = () => {};
  private onTouchedCallback: () => any = () => {};

  constructor(private dataCardService: DataCardService) {
    this.dataCardService.isDataCardClosed.subscribe(isClosed => this.setDisabledState(isClosed));
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.notRelevant) {
      if (this.notRelevant) {
        this.oldValue = this.innerModel;
        this.writeValue(undefined);
      } else {
        this.writeValue(this.oldValue);
      }
    }
  }

  get value(): any {
    return this.innerModel;
  }

  set value(value: any) {
    if (!deepEqual(value, this.innerModel)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value);
      }
    }
  }

  onMouseEnter(): void {
    this.hover = !this.disabled;
  }

  onMouseLeave(): void {
    this.hover = false;
  }

  // Set touched on blur
  onBlur(): void {
    this.onTouchedCallback();
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  setDisabledState(isDisabled: boolean): void {
    this.disabled = isDisabled;
  }

  writeValue(newValue: any): void {
    this.value = newValue;
  }

  makeDirtyable(): void { // on user input we're naturally dirty-able
    this.dirtyAble = true;
  }
}
