import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {GeneralInformationBarComponent} from './general-information-bar.component';
import {AppModule} from '../app.module';
import {of} from 'rxjs';
import {DataCard, ChronicDiseasesSection, OverallAssessmentSectionAllOfPrenatalCareAssessment} from '@nspop/gm-web-facade-api';
import {DataFieldComponent} from '../activities/data-field/data-field.component';
import {MatIconTestingModule} from '@angular/material/icon/testing';
import {ObjectHelper} from '../model-wrappers/objectHelper';
import {DataCardService} from '../api/data-card.service';
import {MatDialog, MatDialogModule} from '@angular/material/dialog';
import {PatientContextService} from '../services/patient-context.service';

describe('GeneralInformationBarComponent', () => {
  let component: GeneralInformationBarComponent;
  let fixture: ComponentFixture<GeneralInformationBarComponent>;

  let dataCardServiceStub = DataCardService;

  beforeEach(async () => {
    dataCardServiceStub = {
      selectedDataCard$: of<DataCard>({
        dirty: false,
        episodeOfCareIdentifier : '1123',
        documentUniqueId: 'asd',
        version: 'asdasd',
        documentStatus: 'ACTIVE',
        recordTarget: {
          identifier: '1008821234',
          name: {
            firstNames: ['Peter'],
            middleNames: [],
            lastName: 'Parker'
          }
        },
        overallAssessmentSection: {
          dirty: true,
          prenatalCareAssessment: {
            assessment: 'this aint good',
            careLevel: OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum.Basic
          }
        },
        multiplePregnancySection: {
          numberOfFetuses: 1,
        },
        chronicDiseasesSection: {
          dirty: true,
          conditions: [
            ChronicDiseasesSection.ConditionsEnum.RespiratorySystem,
            ChronicDiseasesSection.ConditionsEnum.CirculatorySystem,
            ChronicDiseasesSection.ConditionsEnum.Thyroid,
            ChronicDiseasesSection.ConditionsEnum.Diabetes
          ],
        },
      } as any)
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ GeneralInformationBarComponent, DataFieldComponent],
      imports: [AppModule, MatIconTestingModule, MatDialogModule],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub}
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(GeneralInformationBarComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('computes care-level display-text on init', () => {
    expect(component.careLevel).toEqual('1');
  });

  it('computes care-level display-text correctly', () => {
    const componentWithExposedMethod = component as unknown as
      {computeCareLevel(carelevel?: OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum): string};
    expect(componentWithExposedMethod.computeCareLevel(undefined)).toEqual('?');
    expect(componentWithExposedMethod.computeCareLevel(
      OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum.Basic)
    ).toEqual('1');
    expect(componentWithExposedMethod.computeCareLevel(
      OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum.Extended)
    ).toEqual('2');
    expect(componentWithExposedMethod.computeCareLevel(
      OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum.SpecialistMultidisciplinaryTeam
    )).toEqual('3');
    expect(componentWithExposedMethod.computeCareLevel(
      OverallAssessmentSectionAllOfPrenatalCareAssessment.CareLevelEnum.HighRiskPregnancyCare
    )).toEqual('4');
  });

  it('computes special conditions display-text', fakeAsync(() => {
    expect(component.chronicDiseases).toEqual(['Åndedrætssystem', 'Kredsløbssygdomme', 'Skjoldbruskkirtel', 'Diabetes']);
  }));

  it('it enters scroll-mode when window is scrolled', (done) => {
    spyOn(component.headerContainer?.nativeElement, 'getBoundingClientRect').and.returnValue({top: -200});
    expect(component.headerInScrollMode).toBeFalse();
    window.dispatchEvent(new Event('scroll'));
    fixture.detectChanges();
    fixture.whenStable().then(() => {
      expect(component.headerInScrollMode).toBeTrue();
      done();
    });
  });

  it('it leaves scroll-mode when window is scrolled back', (done) => {
    // First let's scroll down
    const returnValue = {top: -200};
    spyOn(component.headerContainer?.nativeElement, 'getBoundingClientRect').and.returnValue(returnValue);
    expect(component.headerInScrollMode).toBeFalse();
    window.dispatchEvent(new Event('scroll'));
    fixture.detectChanges();

    // Cool... Now let's scroll back up
    returnValue.top = 10;
    window.dispatchEvent(new Event('scroll'));
    fixture.detectChanges();

    // Now we'd expect to no longer be in scroll mode
    fixture.whenStable().then(() => {
      expect(component.headerInScrollMode).toBeFalse();
      done();
    });
  });

  it('handles that active datacard could be undefined', fakeAsync(() => {
    component = new GeneralInformationBarComponent({selectedDataCard$: of()} as any, {} as MatDialog , {} as PatientContextService);
    component.ngOnInit();
    tick();
    expect(component.dataCard).toBeUndefined();
  }));

  it('will navigate to "svangerskab" if no ultrasound date of delivery is set', () => {
    component.dataCard = undefined;
    expect(component.expectedDateOfBirth.route).toBe('/svangerskab');
  });

  it('will navigate to "Målinger" if ultrasound date of delivery has been set', () => {
    component.dataCard = ObjectHelper.setOnPath(
      {},
      'ultrasoundSection.expectedDateOfDeliveryFromPrenatalUltrasoundScan',
      {}
    ) as any;
    expect(component.expectedDateOfBirth.route).toBe('/undersøgelser-og-målinger');
  });

  it('computes display text for rhesus type "positive" correctly', fakeAsync(() => {
    expect(component.computeRhesusType('positive')).toBe('Positiv');
  }));

  it('computes display text for rhesus type "negative" correctly', fakeAsync(() => {
    expect(component.computeRhesusType('negative')).toBe('Negativ');
  }));

  it('computes display text for no/unknown rhesus type correctly', fakeAsync(() => {
    tick();
    expect(component.computeRhesusType(undefined)).toBe('?');
  }));
});
