import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';
import {of} from 'rxjs';
import {FormsModule, NgForm} from '@angular/forms';
import {AppModule} from '../../app.module';
import {map} from 'rxjs/operators';
import {By} from '@angular/platform-browser';
import {DataCardService} from '../../api/data-card.service';
import {ObservationService} from '../../api/observation.service';
import {MeasurementsModalComponent} from './measurements-modal.component';
import {CarePlanService} from '../../api/care-plan.service';

describe('MeasurementsModalComponent', () => {
  let component: MeasurementsModalComponent;
  let fixture: ComponentFixture<MeasurementsModalComponent>;

  let dataCardServiceStub: DataCardService;
  let carePlanServiceStub: CarePlanService;
  let observationService: ObservationService;

  beforeEach(async () => {
    dataCardServiceStub = {
      isDataCardClosed: of(false),
      selectedDataCard$: of({
        expectedDateOfDeliverySection: {},
        prenatalRiskAssessmentSection: {
          finalRiskAssessment: {
            multiplicityRiskAssessment: {}
          },
        },
        overallAssessmentSection: {prenatalCareAssessment: {numberOfFetuses: 2}}
      }),
      refresh: () => of(),
      update: () => of(),
    } as any;

    carePlanServiceStub = {
      getActivityById: (activityId: string) => undefined,
    } as any;

    observationService = {
      createUltrasoundObservations: () => of(),
      createRegularObservations: () => of(),
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ MeasurementsModalComponent ],
      imports: [FormsModule, AppModule],
      providers: [
        {provide: DataCardService, useValue: dataCardServiceStub},
        {provide: ObservationService, useValue: observationService},
        {provide: NgForm, useValue: new NgForm([], [])},
      ]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(MeasurementsModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('fetches datacard and resets observationData on init', fakeAsync(() => {
    let accessed = false;
    dataCardServiceStub.selectedDataCard$ = of({multiplePregnancySection: {numberOfFetuses: 3}} as any).pipe(
      map(x => {accessed = true; return x; })
    );

    fixture = TestBed.createComponent(MeasurementsModalComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();

    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    fixture.detectChanges();
    component.ngOnInit();
    tick();
    expect(accessed).toBeTrue();
    expect(component.observationDataRegular?.fetuses?.length).toBe(3);
  }));

  it('allows save if activity was picked', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'regular';
    component.activityId = 'user_selected_some_id_in_form';
    const spy = spyOn(observationService, 'createRegularObservations').and.callThrough();
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).toHaveBeenCalled();
  }));

  it('does not allow save if no activity was picked', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'regular';
    const spy = spyOn(observationService, 'createRegularObservations').and.callThrough();
    component.observationDataRegular = {} as any;
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(spy).not.toHaveBeenCalled();
    const activitySelector = fixture.debugElement.query(By.css('#activitySelector')).nativeElement;
    expect(activitySelector.classList.contains('ng-invalid')).toBeTrue();
  }));

  it('just creates observation when in "regular" mode', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'regular';
    component.activityId = 'user_selected_some_id_in_form';
    const dataCardSpy = spyOn(dataCardServiceStub, 'update').and.callThrough();
    const regularObservationSpy = spyOn(observationService, 'createRegularObservations').and.callThrough();
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(dataCardSpy).not.toHaveBeenCalled();
    expect(regularObservationSpy).toHaveBeenCalled();
  }));

  it('saves datacard and creates observations when in "ultrasound" mode', fakeAsync(() => {
    component.isOpen = true;
    component.ngOnChanges({isOpen: {}} as any);
    component.observationType = 'ultrasound';
    component.activityId = 'user_selected_some_id_in_form';
    const dataCardSpy = spyOn(dataCardServiceStub, 'update').and.callThrough();
    const scanObservationSpy = spyOn(observationService, 'createUltrasoundObservations').and.callThrough();
    fixture.detectChanges();
    component.doSave();
    tick();
    expect(dataCardSpy).toHaveBeenCalled();
    expect(scanObservationSpy).toHaveBeenCalled();
  }));

  it('handles that active datacard could be undefined', fakeAsync(() => {
    component = new MeasurementsModalComponent({} as any, {selectedDataCard$: of()} as any, {} as any);
    component.ngOnInit();
    tick();
    expect(component.dataCard).toBeUndefined();
  }));
});
