import {Component, forwardRef, Input, OnDestroy, OnInit} from '@angular/core';
import {ControlContainer, ControlValueAccessor, NG_VALUE_ACCESSOR, NgForm} from '@angular/forms';
import {DataCard, NumericValueWithUnit, ScanningObservationTable, ScanningObservationTableFetuses} from '@nspop/gm-web-facade-api';
import deepEqual from 'deep-equal';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';
import {ObservationService} from '../../api/observation.service';

type Row = {
  activityId: string;
  date?: Date;
  gestationAge?: string;
  ulWeight?: string;
  weightDeviation?: string;
  fetusPresentation?: string;
  amnioticFluid?: string;
  flow?: string;
  cervix?: string;
  dvp?: string;
  conclusion?: string;
};

export const APP_MEASUREMENTS_ULTRASOUND_CONTROL_VALUE_ACCESSOR: any = {
  provide: NG_VALUE_ACCESSOR,
  useExisting: forwardRef(() => MeasurementsUltrasoundComponent),
  multi: true
};

@Component({
  selector: 'app-measurements-ultrasound',
  templateUrl: './measurements-ultrasound.component.html',
  styleUrls: ['./measurements-ultrasound.component.scss', '../measurements-regular/tables.scss'],
  providers: [APP_MEASUREMENTS_ULTRASOUND_CONTROL_VALUE_ACCESSOR],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class MeasurementsUltrasoundComponent implements ControlValueAccessor, OnInit, OnDestroy {
  public rows: Row[] = [];
  @Input() public showRelatedActivityModal: {isOpen: boolean; activityId?: string; observationType?: 'regular' | 'ultrasound'} | undefined;
  @Input() public measurementsModal: {isOpen: boolean; observationType: 'regular' | 'ultrasound' } | undefined;
  @Input() public saveListeners: (() => any)[] = [];

  private innerModel: DataCardWrapper | undefined;
  private dirtyAble = false;
  private onChangeCallback: (newValue: DataCard|undefined) => any = () => {};
  private onTouchedCallback: () => any = () => {};
  private saveCallback = () => this.buildObservationTable();

  constructor(public observationService: ObservationService) { }

  get value(): DataCardWrapper|undefined {
    return this.innerModel;
  }

  set value(value: DataCardWrapper|undefined) {
    if (!deepEqual(value?.asDataCard, this.innerModel?.asDataCard)) {
      this.innerModel = value;
      if (this.dirtyAble) {
        this.onChangeCallback(value?.asDataCard);
      }
      if (!this.dirtyAble && this.innerModel) { // We don't want to mark the field as dirty while data gets initialized
        this.dirtyAble = true;
      }
    }
  }

  ngOnInit(): void {
    this.saveListeners.push(this.saveCallback);
    this.buildObservationTable();
  }

  ngOnDestroy(): void {
    this.saveListeners = this.saveListeners.splice(this.saveListeners.indexOf(this.saveCallback), 1);
  }

  registerOnChange(fn: any): void {
    this.onChangeCallback = fn;
  }

  registerOnTouched(fn: any): void {
    this.onTouchedCallback = fn;
  }

  writeValue(newValue: DataCard): void {
    this.value = new DataCardWrapper(newValue);
  }

  public computeBirthDate(): Date {
    return new Date();
  }

  public showActivity(activityId: string): void {
    if (this.showRelatedActivityModal) {
      this.showRelatedActivityModal.activityId = activityId;
      this.showRelatedActivityModal.isOpen = true;
      this.showRelatedActivityModal.observationType = 'ultrasound';
    }
  }

  public createMeasurements(): void {
    if (this.measurementsModal) {
      this.measurementsModal.observationType = 'ultrasound';
      this.measurementsModal.isOpen = true;
    }
  }

  private buildObservationTable(): void {
    this.observationService.ultrasoundObservationTable$.subscribe((table: ScanningObservationTable | undefined) => {
      if (!table) { return; }

      this.rows = [];
      const activityGrouping: Record<string, ScanningObservationTable> = {};
      const activityIds: Set<string> = new Set();

      table.mother?.rows?.forEach(row => {
        if (row.activityId) {
          activityIds.add(row.activityId);
        }
      });

      table.fetuses?.forEach(fetus => fetus.rows?.forEach((row) => {
        if (row.activityId) {
          activityIds.add(row.activityId);
        }
      }));

      for (const activityId of activityIds) {
        const mother = {rows: table.mother?.rows?.filter(x => x.activityId === activityId) || []};
        const fetuses = table.fetuses?.map(fetus => ({
          id: fetus.id,
          name: fetus.name,
          rows: fetus.rows?.filter((x) => x.activityId === activityId) || []
        }));
        activityGrouping[activityId] = {mother, fetuses};
      }
      Object.keys(activityGrouping).forEach(activityId => {
        const data = activityGrouping[activityId];
        const row: Row = {
          activityId,
          date: this.extractPropertyFromAny<Date>('date', data),
          gestationAge: this.extractPropertyFromAny<string>('gestationAge', data) || '-',
          ulWeight: this.mergeProperty('ulWeight', data.fetuses || []),
          weightDeviation: this.mergeProperty('weightDeviation', data.fetuses || []),
          fetusPresentation: this.mergeProperty('fetusPresentation', data.fetuses || []),
          amnioticFluid: data.mother?.rows?.[0]?.amnioticFluid || '-',
          flow: data.mother?.rows?.[0]?.flow || '-',
          cervix: this.displayNumericValue(data.mother?.rows?.[0]?.cervix),
          dvp: this.displayNumericValue(data.mother?.rows?.[0]?.dvp),
        };
        this.rows.push(row);
      });
    });
  }

  private extractPropertyFromAny<T>(property: string, table: ScanningObservationTable): T|undefined {
    if (table.mother?.rows) {
      if (Object.keys(table.mother?.rows[0] || {}).includes(property)) {
        const probablyResult = (table.mother?.rows[0] as any)[property];
        if (probablyResult) {
          return probablyResult as T;
        }
      }
    }
    let result: T|undefined;
    table.fetuses?.forEach((fetus) => {
      if ((Object.keys(fetus.rows?.[0] || {}) || []).includes(property)) {
        const probablyResult = (fetus.rows?.[0] as any)[property];
        if (probablyResult) {
          result = probablyResult;
          return;
        }
      }
    });
    return result;
  }

  private displayNumericValue(value?: NumericValueWithUnit): string {
    if (value && value.value) {
      return `${value.value} ${value.unit}`;
    }
    return '-';
  }

  private mergeProperty(property: string, feti: ScanningObservationTableFetuses[]): string {
    return feti.map((fetus) => {
      // @ts-ignore
      const element = fetus?.rows?.[0]?.[property];
      if (!element) {
        return '-';
      }
      if (element.value && element.unit) {
        return this.displayNumericValue(element);
      }
      return element;
    }).reduce((a, b) =>  a + ', ' + b);
  }
}
