import {ErrorHandler, Injectable, NgZone} from '@angular/core';
import {Router} from '@angular/router';
import {PatientContextService} from './patient-context.service';
import {HttpErrorResponse} from '@angular/common/http';
import {deleteCookie} from '../../utils/cookies';
import {environment} from '../../environments/environment';
import {appUrls} from '../../utils/URL_STRING_LITERALS';

@Injectable({
  providedIn: 'root'
})
export class ErrorService implements ErrorHandler {

  constructor(private zone: NgZone,
              private router: Router,
              private patientContextService: PatientContextService) {
  }

  /**
   * Handles app-specific errors
   * @param error any errors thrown
   */
  handleError(error: any): void {
    if (error instanceof CprDoesNotExistError) { this.handleCprDoesNotExistError(); }
    else if (error instanceof PartialDataCardError) { this.handlePartialDataCardError(); }
    else if (error instanceof AppError) { this.handleAppError(); }
    console.error(error);
  }

  private handleCprDoesNotExistError(): void {
    this.patientContextService.setPatientExists(false);
  }

  private handlePartialDataCardError(): void {
    this.patientContextService.setHasPrivatelyMarkedData(true);
    this.router.navigate([appUrls.privatelyMarked]);
  }

  private handleAppError(): void {
    this.zone.run(() => this.router.navigate([appUrls.somethingWentWrong]));
  }

  /**
   * Browser intercepts redirects in xhr calls and returns 200 OK
   * this will check if any login redirects were requested by SAML
   * @param error the httpErrorResponse
   */
  redirectIfRequested(error: HttpErrorResponse): void {
    const redirectHtmlStart = '<html><head><meta http-equiv="refresh" content=';
    // Regex for oiosaml.fragment Cookie: url=(.*?)\".*document\.cookie = '(.*)';

    if (error.status === 200 && error.error.text?.includes(redirectHtmlStart)) {
      console.log('should be redirected');
      const redirectUrlRegex = /url=(.*?)\"/;
      const matches = redirectUrlRegex.exec(error.error.text);
      if (matches !== null && matches.length > 1) {
        deleteCookie('JSESSIONID');
        window.location.assign(environment.base_path);
      }
    }
  }
}

// Custom App Errors
/**
 * The Generic Application error. Parent to all other errors.
 */
export interface GenericAppError {
  errorMessage: string;
  innerError?: Error;
}

/**
 * Top level error
 */
export class AppError implements GenericAppError {
  errorMessage = 'Der skete en ukendt fejl';
  constructor(public innerError?: Error) {
  }
}

/**
 * Expected error for 404
 */
export class PageNotFoundError extends AppError {
  errorMessage = 'Siden kunne ikke findes';
}

/**
 * Expected error for when patient does not exist
 */
export class CprDoesNotExistError extends AppError {
  errorMessage = 'Cpr findes ikke';
}

/**
 * Expected error for when a patient has privately marked data
 */
export class PartialDataCardError extends AppError {
  errorMessage = 'Patienten har privatmarkeret data';
}

/**
 * Expected error for when patient does not exist
 */
export class BadInputError extends AppError {
  errorMessage = 'Input har ikke rigtig form';
}


