import {ComponentFixture, TestBed} from '@angular/core/testing';

import {TabsComponent} from './tabs.component';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {Component, Input} from '@angular/core';
import {RouterTestingModule} from '@angular/router/testing';
import {of} from 'rxjs';
import {ActivatedRoute, NavigationExtras, Router} from '@angular/router';

@Component({
  selector: 'app-semi-fixed-header',
  template: '<ng-content></ng-content>'
})
class MockSemiFixedHeaderComponent {
  // tslint:disable-next-line:no-input-rename
  @Input('fixed-at') fixedAt = 0;
}


describe('TabsComponent', () => {
  let fixture: ComponentFixture<TabsComponent>;
  let component: TabsComponent;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [TabsComponent, MockSemiFixedHeaderComponent],
      imports: [AngularMaterialModule, RouterTestingModule]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(TabsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('is defined', () => {
    expect(component).toBeTruthy();
  });

  it('updates routes query params with tab index', () => {
    let params: any = null;
    const activeRouteMock = {
      queryParamMap: of({has: () => true, get: () => '3'})
    } as unknown as ActivatedRoute;
    const routerMock = {
      navigate: (whatever: any[], options: NavigationExtras) => {
        params = options.queryParams;
        return {then: (x: () => any) => x()};
      }
    } as unknown as Router;
    const tabs = new TabsComponent(activeRouteMock, routerMock);
    tabs.ngOnInit();
    expect(tabs.tabIndex).toBe(3);
    tabs.tabIndex = 1;
    expect(params.tab).toBe(1);
  });

  it('does not alter route if tab-index is changed to be the same as it was', () => {
    const activeRouteMock = {
      queryParamMap: of({has: () => true, get: () => '1'})
    } as unknown as ActivatedRoute;
    const routerMock = {
      navigate: () => ({then: (x: () => any) => x()})
    } as unknown as Router;
    const navSpy = spyOn(routerMock, 'navigate');

    const tabs = new TabsComponent(activeRouteMock, routerMock);
    tabs.ngOnInit();
    expect(tabs.tabIndex).toBe(1);
    tabs.tabIndex = 1;
    expect(navSpy).toHaveBeenCalledTimes(1); // 1 time for initial navigation to 1 - but not again for setting tabindex to 1 again
  });

  it('does not alter route if we have not checked view yet adn we\'re trying to set the tab query-param to undefined', () => {
    const activeRouteMock = {
      queryParamMap: of({has: () => true, get: () => '1'})
    } as unknown as ActivatedRoute;
    const routerMock = {
      navigate: () => ({then: (x: () => any) => x()})
    } as unknown as Router;
    const navSpy = spyOn(routerMock, 'navigate');

    const tabs = new TabsComponent(activeRouteMock, routerMock);
    tabs.ngOnInit();
    expect(tabs.tabIndex).toBe(1);
    tabs.tabIndex = 0;
    expect(navSpy).toHaveBeenCalledTimes(1); // 1 time for initial navigation to 1 - but not again for setting tabindex to 1 again
    (tabs as any).ready = true;
    tabs.tabIndex = 0;
    expect(navSpy).toHaveBeenCalledTimes(2);
  });

  it('starts on the tab index denoted in the "tab" query-param', () => {
    const activeRouteMock = {
      queryParamMap: of({has: () => true, get: () => '3'})
    } as unknown as ActivatedRoute;
    const routerMock = {
      navigate: () => ({
        then: (x: () => any) => x()
      })
    } as unknown as Router;
    const tabs = new TabsComponent(activeRouteMock, routerMock);
    tabs.ngOnInit();
    expect(tabs.tabIndex).toBe(3);
  });

  it('starts at first tab (tabindex=0) of no query-param is present', () => {
    expect(component.tabIndex).toBe(0);
  });
});
