import {DataCard} from '@nspop/gm-web-facade-api';
import {calculateFromAndToWeek, calculatePregnancyStartDate, calculateWeekNumberFromPregnancyStart} from './dates';

describe('Dates Util', () => {
  describe('calculateFromAndToWeek', () => {
    it('returns from and to week as ActivityPlannedTime', () => {
      const dataCard: DataCard = buildStubDataCard(new Date('2021-01-01'), undefined);
      const from = calculateFromAndToWeek(dataCard, 4, 12).from;
      const to = calculateFromAndToWeek(dataCard, 4, 12).to;

      expect(from?.getDate()).toBe(29); // (4 * 7) + 1
      expect(from?.getMonth()).toBe(0);
      expect(from?.getFullYear()).toBe(2021);

      // (12 * 7) + 1 (12 weeks as days + 1 because first of jan) = 85
      // 85 - 31 (minus days in jan) = 54
      // 54 - 28 (minus days in feb2021) = 26
      expect(to?.getDate()).toBe(26);
      // Above calculations gives us +2 to months
      expect(to?.getMonth()).toBe(2);
      expect(to?.getFullYear()).toBe(2021);
    });

    it('returns undefined if dataCard is undefined', () => {
      expect(calculateWeekNumberFromPregnancyStart(undefined, new Date('2021-01-01'))).toBe(undefined);
    });

    it('returns undefined if week date is undefined', () => {
      const dataCard: DataCard = buildStubDataCard(new Date('2021-01-01'), undefined);
      expect(calculateWeekNumberFromPregnancyStart(dataCard, undefined)).toBe(undefined);
    });
  });

  describe('calculateWeekNumberFromPregnancyStart', () => {
    it('calculates week number from pregnancyStart', () => {
      const dataCard: DataCard = buildStubDataCard(new Date('2021-01-01'), undefined);
      expect(calculateWeekNumberFromPregnancyStart(dataCard, new Date('2021-01-01'))).toBe(0);
      expect(calculateWeekNumberFromPregnancyStart(dataCard, new Date('2021-01-31'))).toBe(4);
    });

    it('returns undefined if dataCard is undefined', () => {
      expect(calculateWeekNumberFromPregnancyStart(undefined, new Date('2021-01-01'))).toBe(undefined);
    });

    it('returns undefined if week date is undefined', () => {
      const dataCard: DataCard = buildStubDataCard(new Date('2021-01-01'), undefined);
      expect(calculateWeekNumberFromPregnancyStart(dataCard, undefined)).toBe(undefined);
    });
  });

  describe('calculatePregnancyStartDate', () => {
    it('calculates pregnancyStartDate from lastPeriod when ultraSound is undefined', () => {
      const dataCard: DataCard = buildStubDataCard(new Date('2021-01-01'), undefined);
      expect(calculatePregnancyStartDate(dataCard)?.getDate()).toBe(1);
      expect(calculatePregnancyStartDate(dataCard)?.getMonth()).toBe(0); // months are 0-indexed
      expect(calculatePregnancyStartDate(dataCard)?.getFullYear()).toBe(2021);
    });

    it('calculates pregnancyStartDate from ultraSound when ultraSound is defined', () => {
      const dataCard: DataCard = buildStubDataCard(new Date('2021-01-01'), new Date('2021-10-01'));
      expect(calculatePregnancyStartDate(dataCard)?.getDate()).toBe(1);
      expect(calculatePregnancyStartDate(dataCard)?.getMonth()).toBe(0); // months are 0-indexed
      expect(calculatePregnancyStartDate(dataCard)?.getFullYear()).toBe(2021);
    });

    it('returns pregnancyStartDate = undefined when neither lastPeriod or ultrasound is set', () => {
      const dataCard: DataCard = buildStubDataCard(undefined, undefined);
      expect(calculatePregnancyStartDate(dataCard)).toBe(undefined);
    });
  });
});

function buildStubDataCard(lastPeriod?: Date, expectedDateOfDeliveryFromPrenatalUltrasoundScan?: Date): DataCard {
  return {
    expectedDateOfDeliverySection: {
      expectedDateOfDeliveryFromLastPeriod: {
        lastPeriod
      },
    },
    ultrasoundSection: {
      expectedDateOfDeliveryFromPrenatalUltrasoundScan
    }
  } as DataCard;
}
